# -*- coding: utf-8 -*-

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from persepolis.scripts import logger
from persepolis.constants import OS
from time import sleep
import subprocess
import platform

os_type = platform.system()


def shutDown(parent, gid=None, category=None, password=None):
    # for queue >> gid = None
    # for single downloads >> category = None
    # change value of shutdown in data base
    if category is not None:
        dict = {'category': category,
                'shutdown': 'wait'}

        # update data base
        parent.temp_db.updateQueueTable(dict)
    else:
        # so we have single download
        dict = {'gid': gid,
                'shutdown': 'wait'}

        # update data base
        parent.temp_db.updateSingleTable(dict)

    shutdown_status = "wait"

    while shutdown_status == "wait":
        sleep(5)

        # get shutdown status from data_base
        if category is not None:
            dict = parent.temp_db.returnCategory(category)
        else:
            dict = parent.temp_db.returnGid(gid)

        shutdown_status = dict['shutdown']

    if shutdown_status == "shutdown":

        logger.sendToLog("Shutting down in a minute", "INFO")
        # Make sure all download progresses are stopped.
        parent.stopAllDownloads()

        sleep(20)

        # Make sure all sessions have ended.
        while parent.download_sessions_list:
            sleep(0.5)

        # shutdown_notification = 0 >> persepolis running , 1 >> persepolis is
        # ready for close(closeEvent called) , 2 >> OK, let's close application!
        parent.changeShutdownValue(1)
        while parent.returnShutDownValue() != 2:
            sleep(0.1)

        # close data bases connections
        for db in parent.persepolis_db, parent.plugins_db, parent.temp_db:
            db.closeConnections()

        for i in parent.threadPool:
            i.quit()
            i.wait()

        parent.cleanTempFolder()

        if os_type == OS.LINUX:

            pipe = subprocess.Popen(['sudo', '-S', 'poweroff'],
                                    stdout=subprocess.DEVNULL,
                                    stdin=subprocess.PIPE,
                                    stderr=subprocess.DEVNULL,
                                    shell=False)

            pipe.communicate(password.encode())

        elif os_type == OS.DARWIN:

            pipe = subprocess.Popen(['sudo', '-S', 'shutdown', '-h', 'now'],
                                    stdout=subprocess.DEVNULL,
                                    stdin=subprocess.PIPE,
                                    stderr=subprocess.DEVNULL,
                                    shell=False)

            pipe.communicate(password.encode())

        elif os_type == OS.WINDOWS:

            CREATE_NO_WINDOW = 0x08000000
            subprocess.Popen(['shutdown', '-S'],
                             stderr=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stdin=subprocess.PIPE,
                             shell=False,
                             creationflags=CREATE_NO_WINDOW)

        elif os_type in OS.BSD_FAMILY:

            pipe = subprocess.Popen(['sudo', '-S', 'shutdown', '-p', 'now'],
                                    stdout=subprocess.DEVNULL,
                                    stdin=subprocess.PIPE,
                                    stderr=subprocess.DEVNULL,
                                    shell=False)

            pipe.communicate(password.encode())



# -*- coding: utf-8 -*-
"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from time import sleep
from persepolis.scripts import logger
from persepolis.scripts.download_link import DownloadLink
from persepolis.scripts.useful_tools import muxer
try:
    from PySide6.QtCore import QThread, Signal
except:
    from PyQt5.QtCore import QThread
    from PyQt5.QtCore import pyqtSignal as Signal

try:
    from persepolis.scripts import ytdlp_downloader
except ModuleNotFoundError:
    # if youtube_dl module is not installed:
    logger.sendToLog(
        "yt-dlp is not installed.", "ERROR")


# Persepolis download audio and video separately and the muxing them :)
# VideoFinder do this job for Persepolis.
# see data_base.py for understanding the code
# we have video_finder_db_table in data base. it's contains some items that helps
# VideoFinder for managing the situation.
# video_gid >> GID of video link
# audio_gid >> GID of audio link
# video_completed >> Is video downloaded completely?
# audio_completed >> Is audio downloaded completely?
# checking >> VideoFinder must checking or not!


class VideoFinder(QThread):
    VIDEOFINDERCOMPLETED = Signal(dict)

    def __init__(self, video_finder_dictionary, main_window):
        QThread.__init__(self)
        self.main_window = main_window
        self.video_finder_dictionary = video_finder_dictionary

    # First: Download video
    # Second: Download audio
    # Third: Mux video and audio
    def run(self):
        self.video_completed = self.video_finder_dictionary['video_completed']
        self.audio_completed = self.video_finder_dictionary['audio_completed']
        self.muxing = 'no'
        self.checking = 'no'
        self.active = 'yes'

        video_gid = self.video_finder_dictionary['video_gid']
        audio_gid = self.video_finder_dictionary['audio_gid']

        # find category
        dictionary = self.main_window.persepolis_db.searchGidInDownloadTable(video_gid)
        category = dictionary['category']

        # VideoFinder handles downloads by itself, if category is "Single Downloads"
        if category == 'Single Downloads':

            # create an item for this thread in temp_db if not exists!
            try:
                video_finder_plus_gid = 'video_finder_' + str(video_gid)
                self.main_window.temp_db.insertInQueueTable(video_finder_plus_gid)
            except:
                # release lock
                self.main_window.temp_db.lock = False

            # check start time and end time
            add_link_dictionary = self.main_window.persepolis_db.searchGidInAddLinkTable(video_gid)
            start_time = add_link_dictionary['start_time']

            if self.video_completed == 'no' and start_time:

                # set start time only for video and cancel start time for audio.
                # because video will downloaded first and start time must be set for first link! not second one
                self.main_window.persepolis_db.setDefaultGidInAddlinkTable(audio_gid, start_time=True)

        # update checking status in data base for starting the job!
        self.checking = 'yes'
        self.video_finder_dictionary['checking'] = 'yes'

        self.main_window.persepolis_db.updateVideoFinderTable([self.video_finder_dictionary])

        # if category "Single Downloads" >> manage download yourself.
        # if category is not "Single Download" >> just check the status time to time and wait until download ends!
        if self.video_completed == 'no':
            if category == "Single Downloads":

                # start video downloading
                # get add_link_dictionary for video
                add_link_dictionary = self.main_window.persepolis_db.searchGidInAddLinkTable(video_gid)
                # create download_session
                video_download_session = ytdlp_downloader.Ytdp_Download(add_link_dictionary, self.main_window, video_gid)

                # add download_session and gid to download_session_dict
                download_session_dict = {'gid': video_gid,
                                         'download_session': video_download_session}

                # append download_session_dict to download_sessions_list
                self.main_window.download_sessions_list.append(download_session_dict)

                # strat download in thread
                new_download = DownloadLink(video_gid, video_download_session, self.main_window)
                self.main_window.threadPool.append(new_download)
                self.main_window.threadPool[-1].start()

            # check the download status
            # continue loop and check the download status
            # if checking == 'no' >> problem has been occurred and download has been canceled.
            while self.video_completed != 'yes' and self.checking == 'yes':

                sleep(1)

        if self.video_completed == 'yes':

            if self.video_finder_dictionary['video_completed'] == 'no':

                # update data base
                self.video_finder_dictionary['video_completed'] = 'yes'

                self.main_window.persepolis_db.updateVideoFinderTable([self.video_finder_dictionary])

            # video is downloaded completely!
            # let's start audio downloading
            if self.audio_completed == 'no':

                # if category "Single Downloads" >> start download yourself.
                # if category is not "Single Download" >> just check the status time to time
                if category == "Single Downloads":
                    # get add_link_dictionary for video
                    add_link_dictionary = self.main_window.persepolis_db.searchGidInAddLinkTable(audio_gid)
                    # create download_session
                    audio_download_session = ytdlp_downloader.Ytdp_Download(add_link_dictionary, self.main_window, audio_gid)

                    # add download_session and gid to download_session_dict
                    download_session_dict = {'gid': audio_gid,
                                             'download_session': audio_download_session}

                    # append download_session_dict to download_sessions_list
                    self.main_window.download_sessions_list.append(download_session_dict)

                    # set speed limitation of video_download_session for audio_download_session
                    # audio_download_session.sleep_for_speed_limiting = video_download_session.sleep_for_speed_limiting
                    # strat download in thread
                    new_download = DownloadLink(audio_gid, audio_download_session, self.main_window)
                    self.main_window.threadPool.append(new_download)
                    self.main_window.threadPool[-1].start()

                # check the download status
                # continue loop and check the download status
                # if checking == 'no' >> problem occurred and downloading canceled.
                while self.audio_completed != 'yes' and self.checking == 'yes':
                    sleep(1)

        self.checking = 'no'

        # lets start muxing!
        if self.video_completed == 'yes' and self.audio_completed == 'yes':

            audio_file_exists = False
            video_file_exists = False
            # wait until the data_base is updated
            while not (audio_file_exists) or not (video_file_exists):
                sleep(0.5)
                # checking for file existance
                # find file path
                video_file_dictionary = self.main_window.persepolis_db.searchGidInAddLinkTable(video_gid)
                audio_file_dictionary = self.main_window.persepolis_db.searchGidInAddLinkTable(audio_gid)

                # find inputs and output file path for ffmpeg
                video_file_path = video_file_dictionary['download_path']
                audio_file_path = audio_file_dictionary['download_path']
                video_file_exists = os.path.isfile(video_file_path)
                audio_file_exists = os.path.isfile(audio_file_path)

            self.video_finder_dictionary['audio_completed'] = 'yes'
            self.video_finder_dictionary['checking'] = 'no'
            self.video_finder_dictionary['muxing_status'] = 'started'

            self.muxing = 'started'

            # update data base
            self.main_window.persepolis_db.updateVideoFinderTable([self.video_finder_dictionary])

            # audio and video files are downloaded completely.
            # lets start muxing
            result_dictionary = muxer(self.main_window, self.video_finder_dictionary)
            error_message = result_dictionary['error']
            ffmpeg_error_message = result_dictionary['ffmpeg_error_message']

            if ffmpeg_error_message:
                logger.sendToLog('ffmpeg error: ' + str(ffmpeg_error_message), 'DOWNLOAD ERROR')

            if error_message == 'no error':
                self.video_finder_dictionary['muxing_status'] = 'complete'
                self.muxing = 'complete'
            else:
                self.video_finder_dictionary['muxing_status'] = 'error'
                self.muxing = 'error'

            # update data base
            self.main_window.persepolis_db.updateVideoFinderTable([self.video_finder_dictionary])

            complete_dictionary = {'error': error_message,
                                   'final_path': result_dictionary['final_path'],
                                   'final_size': result_dictionary['final_size'],
                                   'video_gid': self.video_finder_dictionary['video_gid'],
                                   'audio_gid': self.video_finder_dictionary['audio_gid'],
                                   'download_path': self.video_finder_dictionary['download_path'],
                                   'category': category}

            # emit error_message
            self.VIDEOFINDERCOMPLETED.emit(complete_dictionary)

        self.active = 'no'

        if category == 'Single Downloads':

            # check if user selected shutdown after download in progress window.
            shutdown_dict = self.main_window.temp_db.returnCategory(video_finder_plus_gid)
            shutdown_status = shutdown_dict['shutdown']

            if shutdown_status == 'wait':

                # it means user want to persepolis shutdown system after download.
                # write 'shutdown' value for this category in temp_db
                shutdown_dict = {'category': video_finder_plus_gid,
                                 'shutdown': 'shutdown'}
                self.main_window.temp_db.updateQueueTable(shutdown_dict)



# -*- coding: utf-8 -*-

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from time import sleep
from persepolis.scripts import logger
from persepolis.scripts.bubble import notifySend
from persepolis.scripts import persepolis_lib_prime
from persepolis.scripts.video_finder import VideoFinder
from persepolis.scripts.download_link import DownloadLink

try:
    from PySide6.QtCore import QThread, Signal, QCoreApplication
except:
    from PyQt5.QtCore import QThread, QCoreApplication
    from PyQt5.QtCore import pyqtSignal as Signal
try:
    from persepolis.scripts import ytdlp_downloader
except ModuleNotFoundError:
    # if youtube_dl module is not installed:
    logger.sendToLog(
        "yt-dlp is not installed.", "ERROR")


class Queue():
    def __init__(self, queue_dict, main_window):
        self.queue_name = queue_dict['category']
        self.start_time_enable = queue_dict['start_time_enable']
        self.start_time = queue_dict['start_time']
        self.end_time_enable = queue_dict['end_time_enable']
        self.end_time = queue_dict['end_time']
        self.reverse = queue_dict['reverse']
        self.limit_enable = queue_dict['limit_enable']
        self.limit_value = queue_dict['limit_value']
        self.after_download = queue_dict['after_download']
        self.gid_list = queue['gid_list']
        self.main_window = main_window

    def start(self):




# -*- coding: utf-8 -*-

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import time
from time import sleep
from persepolis.scripts import logger
from persepolis.scripts import persepolis_lib_prime
from persepolis.scripts.video_finder import VideoFinder
from persepolis.scripts.download_link import DownloadLink

try:
    from PySide6.QtCore import QThread, Signal, QCoreApplication
except:
    from PyQt5.QtCore import QThread, QCoreApplication
    from PyQt5.QtCore import pyqtSignal as Signal
try:
    from persepolis.scripts import ytdlp_downloader
except ModuleNotFoundError:
    # if youtube_dl module is not installed:
    logger.sendToLog(
        "yt-dlp is not installed.", "ERROR")


# this thread is managing queue
class Queue(QThread):
    # this signal emitted when download status of queue changes to stop
    REFRESHTOOLBARSIGNAL = Signal(str)
    NOTIFYSENDSIGNAL = Signal(list)

    def __init__(self, category, start_time, end_time, parent):
        QThread.__init__(self)
        self.category = str(category)
        self.main_window = parent
        self.start_time = start_time
        self.end_time = end_time

    def run(self):
        self.start = True
        self.stop = False
        self.limit_changed = False
        self.after = False
        self.break_for_loop = False

        queue_counter = 0

        # this list contains gid_list of all active video finder in queue.
        video_finder_list = []

        # queue repeats 5 times!
        # and every time loads queue list again!
        # It is helps for checking new downloads in queue
        # and retrying for failed downloads.
        for counter in range(5):

            # read downloads information from data base
            download_table_dict = self.main_window.persepolis_db.returnItemsInDownloadTable(self.category)
            category_table_dict = self.main_window.persepolis_db.searchCategoryInCategoryTable(self.category)

            gid_list = category_table_dict['gid_list']

            # sort downloads top to the bottom of the list OR bottom to the top
            if not (self.main_window.reverse_checkBox.isChecked()):
                gid_list.reverse()

            # check that if user set start time
            if self.start_time and counter == 0:

                # find first download
                # set start time for first download in queue
                # status of first download must not be complete
                for gid in gid_list:

                    # get download information dictionary
                    dictionary = download_table_dict[gid]

                    # find status of download
                    status = dictionary['status']

                    if status != 'complete':
                        # We find first item! GREAT!
                        add_link_dict = {'gid': gid}

                        # set start_time for this download
                        add_link_dict['start_time'] = self.start_time

                        # write changes in data base
                        self.main_window.persepolis_db.updateAddLinkTable([add_link_dict])

                        # delete add_link_dict
                        del add_link_dict

                        # job is done! break the loop
                        break

            for gid in gid_list:
                # if gid is related to video finder, so start  Video Finder thread for checking status
                # check video_finder_threads_dict, perhaps a thread started before for this gid
                if (gid in self.main_window.all_video_finder_gid_list):
                    video_finder_link = True

                    video_finder_dictionary = self.main_window.persepolis_db.searchGidInVideoFinderTable(gid)

                    if video_finder_dictionary['video_gid'] not in self.main_window.video_finder_threads_dict.keys():

                        # start new video finder thread
                        video_finder_gid_list = [video_finder_dictionary['video_gid'],
                                                 video_finder_dictionary['audio_gid']]

                        new_video_finder = VideoFinder(video_finder_dictionary, self.main_window)
                        self.main_window.threadPool.append(new_video_finder)
                        self.main_window.threadPool[-1].start()
                        self.main_window.threadPool[-1].VIDEOFINDERCOMPLETED.connect(self.main_window.videoFinderCompleted)

                        # add thread to video_finder_threads_dict
                        self.main_window.video_finder_threads_dict[video_finder_dictionary['video_gid']] = new_video_finder

                        video_finder_list.append(video_finder_gid_list)
                else:
                    video_finder_link = False

                add_link_dict = {'gid': gid}

                # find download information
                dictionary = download_table_dict[gid]

                # if download was completed, continue the loop
                # with the next iteration of the loop!
                # We don't want to download it two times :)
                if dictionary['status'] == 'complete':
                    continue

                queue_counter = queue_counter + 1

                # change status of download to waiting
                status = 'waiting'
                dictionary['status'] = status

                if self.end_time:

                    # it means user was set end time for download
                    # set end_hour and end_minute
                    add_link_dict['end_time'] = self.end_time

                # user can set sleep time between download items in queue.
                # see preferences window!
                # find wait_queue value
                wait_queue_list = self.main_window.persepolis_setting.value('settings/wait-queue')
                wait_queue_hour = int(wait_queue_list[0])
                wait_queue_minute = int(wait_queue_list[1])

                # check if user set sleep time between downloads in queue in setting window.
                # if queue_counter is 1 , it means we are in the first download item in queue.
                # and no need to wait for first item.
                if (wait_queue_hour != 0 or wait_queue_minute != 0) and queue_counter != 1:
                    now_time_hour = int(time.strftime("%H"))
                    now_time_minute = int(time.strftime("%M"))
                    now_time_second = int(time.strftime("%S"))

                    # add extra minute if we are in second half of minute
                    if now_time_second > 30:
                        now_time_minute = now_time_minute + 1

                    # hour value can not be more than 23 and minute value can not be more than 59.
                    sigma_minute = wait_queue_minute + now_time_minute
                    sigma_hour = wait_queue_hour + now_time_hour
                    if sigma_minute > 59:
                        sigma_minute = sigma_minute - 60
                        sigma_hour = sigma_hour + 1

                    if sigma_hour > 23:
                        sigma_hour = sigma_hour - 24

                    # setting sigma_hour and sigma_minute for download's start time!
                    add_link_dict['start_time'] = str(sigma_hour) + ':' + str(sigma_minute)

                # write changes in data base
                self.main_window.persepolis_db.updateAddLinkTable([add_link_dict])
                add_link_dict = self.main_window.persepolis_db.searchGidInAddLinkTable(gid)

                if video_finder_link:
                    # create video download_session
                    download_session = ytdlp_downloader.Ytdp_Download(add_link_dict, self.main_window, gid)
                else:
                    # create download_session
                    # check if it's single_video_link or not
                    answer_dictionary = self.main_window.persepolis_db.searchGidInVideoFinderTable2(gid)
                    if answer_dictionary is not None:
                        single_video_link = True
                    else:
                        single_video_link = False

                    if not (single_video_link):
                        download_session = persepolis_lib_prime.Download(add_link_dict, self.main_window, gid)

                        # check limit speed value
                        download_session.limitSpeed(self.main_window.limit_dial.value())
                    else:
                        download_session = ytdlp_downloader.Ytdp_Download(add_link_dict, self.main_window, gid, single_video_link=True)
                        # add gid to single_video_link_gid_list
                        self.main_window.single_video_link_gid_list.append(gid)

                # add download_session and gid to download_session_dict
                download_session_dict = {'gid': gid,
                                         'download_session': download_session}

                # append download_session_dict to download_sessions_list
                self.main_window.download_sessions_list.append(download_session_dict)

                # strat download in thread
                new_download = DownloadLink(gid, download_session, self.main_window)
                self.main_window.threadPool.append(new_download)
                self.main_window.threadPool[-1].start()

                # delete add_link_dict
                del add_link_dict
                sleep(3)

                # continue loop until download has finished
                while status == 'downloading' or status == 'waiting' or status == 'paused' or status == 'scheduled':

                    sleep(1)
                    dictionary = self.main_window.persepolis_db.searchGidInDownloadTable(gid)

                    status = dictionary['status']

                    if status == 'error':
                        error = 'error'
                        # write error_message in log file
                        error_message = 'Download failed - GID : '\
                            + str(gid)\
                            + '- Message : '\
                            + error

                        logger.sendToLog(error_message, 'DOWNLOAD ERROR')

                    elif status == 'complete':
                        complete_message = 'Download complete - GID : '\
                            + str(gid)

                        # write in log the complete_message
                        logger.sendToLog(complete_message, 'DOWNLOADS')

                        # check that is this related to video finder thread or not.
                        if gid in self.main_window.all_video_finder_gid_list:

                            # find related thread
                            for list in video_finder_list:

                                if gid in list:

                                    video_gid = list[0]

                                    if video_gid in self.main_window.video_finder_threads_dict:
                                        video_finder_thread = self.main_window.video_finder_threads_dict[video_gid]

                                        # check the video and audio and muxing_status
                                        if video_finder_thread.video_completed == 'yes' and video_finder_thread.audio_completed == 'yes':

                                            # wait until end of muxing
                                            while video_finder_thread.active == 'yes':

                                                sleep(0.5)

                                break

                    if self.stop:
                        # it means user stopped queue
                        # search gid in download_sessions_list
                        for download_session_dict in self.main_window.download_sessions_list:
                            if download_session_dict['gid'] == gid:
                                # stop download
                                download_session_dict['download_session'].downloadStop()
                                break

                    if status == 'downloading' and self.limit_changed:
                        # It means user want to limit download speed
                        # get limitation value
                        limit_value = self.main_window.limit_dial.value()

                        # apply limitation
                        for download_session_dict in self.main_window.download_sessions_list:
                            if download_session_dict['gid'] == gid:

                                download_session_dict['download_session'].limitSpeed(limit_value)
                                break

                        # done!
                        self.limit_changed = False

                # it means queue stopped at end time or user stopped queue
                if status == 'stopped':

                    for video_finder_gid_list in video_finder_list:

                        video_gid = video_finder_gid_list[0]

                        video_finder_dictionary = self.main_window.persepolis_db.searchGidInVideoFinderTable(video_gid)

                        if video_finder_dictionary:

                            # tell video finder thread to stop checking
                            if video_finder_dictionary['video_completed'] == 'no' or video_finder_dictionary['audio_completed'] == 'no':

                                video_finder_dictionary['checking'] = 'no'
                                self.main_window.persepolis_db.updateVideoFinderTable([video_finder_dictionary])

                                video_finder_thread = self.main_window.video_finder_threads_dict[video_gid]
                                video_finder_thread.checking = 'no'

                            elif not (self.stop) and self.after and video_finder_dictionary['muxing_status'] == 'started':
                                # downloads were completed and video finder started Muxing
                                # wait until the end of muxing
                                # don't turn of the computer.
                                # video finder will be deleted from data base when muxing ended.
                                # so check data base every second

                                video_finder_thread = self.main_window.video_finder_threads_dict[video_finder_dictionary['video_gid']]

                                while video_finder_thread.active == 'yes':
                                    sleep(1)

                    if self.stop and self.after:
                        # It means user activated shutdown before and now user
                        # stopped queue . so after download must be canceled
                        self.main_window.after_checkBox.setChecked(False)

                    self.stop = True
                    self.limit_changed = False

                    # it means that break outer "for" loop
                    self.break_for_loop = True

                    if str(self.main_window.category_tree.currentIndex().data()) == str(self.category):
                        self.REFRESHTOOLBARSIGNAL.emit(self.category)

                    # show notification
                    self.NOTIFYSENDSIGNAL.emit([QCoreApplication.translate("mainwindow_src_ui_tr", "Persepolis"),
                                                QCoreApplication.translate("mainwindow_src_ui_tr", "Queue Stopped!"),
                                                10000, 'no'])

                    # write message in log
                    logger.sendToLog('Queue stopped', 'DOWNLOADS')

                    break

            if self.break_for_loop:
                break

        if self.start:
            # if queue finished :
            self.start = False

            # this section is sending shutdown signal to the shutdown script(if user
            # select shutdown for after download)
            if self.after:
                # write 'shutdown' value for this category in temp_db
                shutdown_dict = {'category': self.category, 'shutdown': 'shutdown'}
                self.main_window.temp_db.updateQueueTable(shutdown_dict)

                # show a notification about system is shutting down now!
                self.NOTIFYSENDSIGNAL.emit([QCoreApplication.translate("mainwindow_src_ui_tr", 'Persepolis is shutting down'),
                                            QCoreApplication.translate("mainwindow_src_ui_tr", 'your system in 20 seconds'),
                                            15000, 'warning'])

            # show notification for queue completion
            self.NOTIFYSENDSIGNAL.emit([QCoreApplication.translate("mainwindow_src_ui_tr", "Persepolis"),
                                        QCoreApplication.translate("mainwindow_src_ui_tr", 'Queue completed!'),
                                        10000, 'queue'])

            # write a message in log
            logger.sendToLog('Queue completed', 'DOWNLOADS')

            self.stop = True
            self.limit_changed = False
            self.after = False

            if str(self.main_window.category_tree.currentIndex().data()) == str(self.category):
                self.REFRESHTOOLBARSIGNAL.emit(self.category)



#!/usr/bin/env python3
# coding: utf-8
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import os
import platform

# finding os platform
os_type = platform.system()

# Checking dependencies!
not_installed = ''

# PyQt5 or PySide6
try:
    import PySide6
    print('python3-pyside6 is found')
    pyside6_is_installed = True
except:
    pyside6_is_installed = False

if not(pyside6_is_installed):
    try:
        import PyQt5
        print('python3-pyqt5 is found')
    except:
        print('Error : python3-pyqt5 or pyside6 must be installed!')
        not_installed = not_installed + '(PyQt5 or PySide6) '

# python3-requests
try:
    import requests
    print('python3-requests is found!')
except:
    print('Error : requests is not installed!')
    not_installed = not_installed + 'python3-requests, '

# python3-urllib3
try:
    import urllib3
    print('python3-urllib3 is found!')
except:
    print('Error : urllib3 is not installed!')
    not_installed = not_installed + 'python3-urllib3, '


# python3-setproctitle
try:
    import setproctitle
    print('python3-setproctitle is found!')
except:
    print("Warning: setproctitle is not installed!")
    not_installed = not_installed + 'python3-setproctitle, '

# python3-PySocks
try:
    import socks
    print('python3-pysocks is found!')
except:
    print("Warning: python3-pysocks is not installed!")
    not_installed = not_installed + 'python3-pysocks, '


# psutil
try:
    import psutil
    print('python3-psutil is found!')
except:
    print("Warning: python3-psutil is not installed!")
    not_installed = not_installed + 'psutil, '

# yt_dlp
try:
    import yt_dlp
    print('yt-dlp is found')
except:
    print('Warning: yt-dlp is not installed!')
    not_installed = not_installed + 'yt-dlp, '

# ffmpeg
answer = os.system('ffmpeg -version 1>/dev/null')
if answer != 0:
    print("Warning: ffmpeg not installed!")
    not_installed = not_installed + 'ffmpeg, '
else:
    print('ffmpeg is found!')


if os_type == 'Linux':
    try:
        from dasbus.connection import SessionMessageBus
        print('python3-dasbus is found!')
    except:
        print('python3-dasbus is not installed!')
        not_installed = not_installed + 'python3-dasbus,'

if not_installed != '':
    print('########################')
    print('####### WARNING ########')
    print('########################')
    print('Some dependencies are not installed .It causes some problems for persepolis! : \n')
    print(not_installed + '\n\n')
    print('Read this link for more information: \n')
    print('https://github.com/persepolisdm/persepolis/wiki/git-installation-instruction\n\n')



# -*- coding: utf-8 -*-
"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import requests
try:
    from PySide6.QtCore import QThread, Signal
except:
    from PyQt5.QtCore import QThread
    from PyQt5.QtCore import pyqtSignal as Signal

from persepolis.scripts import logger


# this thread starts download.
class DownloadLink(QThread):
    def __init__(self, gid, download_session, main_window):
        QThread.__init__(self)
        self.gid = gid
        self.download_session = download_session
        self.main_window = main_window

    def run(self):
        # add gid of download to the active gids in temp_db
        # or update data base , if it was existed before
        try:
            self.main_window.temp_db.insertInSingleTable(self.gid)
        except:
            # release lock
            self.main_window.temp_db.lock = False
            dictionary = {'gid': self.gid, 'status': 'active'}
            self.main_window.temp_db.updateSingleTable(dictionary)

        self.download_session.start()


class DownloadSingleLink(QThread):
    DOWNLOADSTATUSSIGNAL = Signal(bool)

    def __init__(self, download_link, file_path):
        QThread.__init__(self)
        self.download_link = download_link
        self.file_path = file_path

    def run(self):
        try:
            # download link
            response = requests.get(self.download_link)
            # write it to file
            with open(self.file_path, 'wb') as f:
                f.write(response.content)

            if response.ok:
                log_message = 'Download complete! ' + str(self.file_path)
                logger.sendToLog(log_message, "INFO")
                self.DOWNLOADSTATUSSIGNAL.emit(True)
        except Exception as e:
            error_message = 'Download was unsuccessful:\n' + str(self.file_path) + '\n' + str(e)
            logger.sendToLog(error_message, "ERROR")
            self.DOWNLOADSTATUSSIGNAL.emit(False)
